"""Python environment module."""

from dataclasses import dataclass
from operator import itemgetter
from pathlib import Path


@dataclass
class PyEnv:
    """Python environment generic class."""

    name: str
    path: str

    def __eq__(self, item):
        return self.path == str(item)


class UserPyEnv(PyEnv):
    """User python environment."""

    scope = 'USER'


class TeamPyEnv(PyEnv):
    """Team python environment."""

    scope = 'TEAM'


class GlobalPyEnv(PyEnv):
    """Global python environment."""

    scope = 'GLOBAL'


def _micromamba_path(path: str) -> Path:
    """Micromamba path environments locations."""
    return Path(f'/micromamba/{path}/envs')


def _micromamba_envs(path, excluded=None) -> list:
    """List micromamba environment list."""
    if excluded is None:
        excluded = []
    elif isinstance(excluded, str):
        excluded = [excluded]

    envs = _micromamba_path(path)
    return sorted(
        [(f.name, f.absolute()) for f in envs.iterdir() if f.is_dir() and f.name not in excluded],
        key=itemgetter(0),
    )


def get_pyenv_user(username: str) -> list:
    """List of python environment available to the user."""
    return [UserPyEnv(*env) for env in _micromamba_envs(username)]


def get_pyenv_team(username: str) -> list:
    """List of python environment available to the user's team."""
    teams = []  # FIXME: pull user teams list from groups
    return [TeamPyEnv(*env) for team in teams for env in _micromamba_envs(f'teams/{team}')]


def get_pyenv_global() -> list:
    """List of python environment available globally."""
    return [GlobalPyEnv(*env) for env in _micromamba_envs('operator', excluded='glicid-jupyterhub')]


def get_pyenv(username: str) -> list:
    """List of all the python environment available to a user."""
    return get_pyenv_user(username) + get_pyenv_team(username) + get_pyenv_global()
